package com.fzm.admin.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.lang.Assert;
import com.fzm.admin.params.AdminRoleParam;
import com.fzm.common.annotation.Authentication;
import com.fzm.common.entity.Admin;
import com.fzm.common.entity.AdminRole;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.model.ResponseModel;
import com.fzm.common.service.AdminRoleService;
import com.fzm.common.service.AdminService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author wt
 * @date 2022/3/11
 */
@Authentication
@RestController
@RequestMapping(value = "/admin/role")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Api(tags = "管理员权限管理")
public class AdminRoleController {
    private final AdminService adminService;
    private final AdminRoleService adminRoleService;
    private final PasswordEncoder passwordEncoder;


    @GetMapping("/list")
    @ApiOperation(value = "查询所有权限")
    public ResponseModel<List<AdminRole>> list() {
        return ResponseModel.success(adminRoleService.listAll());
    }

    @PostMapping("/add")
    @ApiOperation("新增权限")
    public ResponseModel<Object> add(@RequestBody AdminRoleParam adminRoleParam) {
        String nickname = adminRoleParam.getNickname();
        Assert.notBlank(nickname, "昵称不能为空");

        String username = adminRoleParam.getUsername();
        Assert.notBlank(username, "用户名不能为空");

        String password = adminRoleParam.getPassword();
        Assert.notBlank(password, "密码不能为空");

        Integer roleId = adminRoleParam.getRoleId();
        Assert.notNull(roleId, "权限ID不能为空");
        List<AdminRole> adminRoles = adminRoleService.listAll();
        Assert.isTrue(adminRoles.stream().map(AdminRole::getRoleId).anyMatch(id -> id.equals(roleId)), "请选择正确的权限");

        Admin admin = BeanUtil.copyProperties(adminRoleParam, Admin.class);
        admin.setPassword(passwordEncoder.encode(password));

        adminService.save(admin);

        return ResponseModel.success();
    }

    @PostMapping("/update")
    @ApiOperation("编辑权限")
    public ResponseModel<Object> update(@RequestBody AdminRoleParam adminRoleParam) {
        String username = adminRoleParam.getUsername();
        Assert.notBlank(username, "用户名不能为空");

        String password = adminRoleParam.getPassword();
        if (StringUtils.isNotBlank(password)) {
            adminRoleParam.setPassword(passwordEncoder.encode(password));
        }

        Integer roleId = adminRoleParam.getRoleId();
        if (roleId != null) {
            List<AdminRole> adminRoles = adminRoleService.listAll();
            Assert.isTrue(adminRoles.stream().map(AdminRole::getRoleId).anyMatch(id -> id.equals(roleId)), "请选择正确的权限");
        }

        if (StringUtils.isAnyBlank(adminRoleParam.getNickname(), password) && roleId == null) {
            throw GlobalException.newException(ResultCode.VALIDATE_FAILED);
        }

        Admin admin = BeanUtil.copyProperties(adminRoleParam, Admin.class);

        adminService.updateByUserNameIf(admin);

        return ResponseModel.success();
    }

}
